@@ -12,14 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# pylint: disable=too-many-lines
+
 """Code to interface with the `vis/` (the JS viewer).
 """
 from __future__ import annotations
 
-import asyncio
+import base64
 import enum
 import os
-import re
 import sys
 import threading
 import time
@@ -27,14 +28,9 @@ import warnings
 from pathlib import Path
 from typing import Any, Dict, Optional, Tuple
 
+import cv2
 import numpy as np
 import torch
-from aiortc import (
-    RTCConfiguration,
-    RTCIceServer,
-    RTCPeerConnection,
-    RTCSessionDescription,
-)
 from cryptography.utils import CryptographyDeprecationWarning
 from rich.console import Console
 
@@ -50,11 +46,7 @@ from nerfstudio.utils.decorators import check_main_thread, decorate_all
 from nerfstudio.utils.io import load_from_json, write_to_json
 from nerfstudio.utils.writer import GLOBAL_BUFFER, EventName, TimeWriter
 from nerfstudio.viewer.server.subprocess import run_viewer_bridge_server_as_subprocess
-from nerfstudio.viewer.server.utils import (
-    force_codec,
-    get_intrinsics_matrix_and_camera_to_world_h,
-)
-from nerfstudio.viewer.server.video_stream import SingleFrameStreamTrack
+from nerfstudio.viewer.server.utils import get_intrinsics_matrix_and_camera_to_world_h
 from nerfstudio.viewer.server.visualizer import Viewer
 
 warnings.filterwarnings("ignore", category=CryptographyDeprecationWarning)
@@ -70,11 +62,13 @@ def get_viewer_version() -> str:
 
 
 @check_main_thread
-def setup_viewer(config: cfg.ViewerConfig, log_filename: Path, datapath: str):
+def setup_viewer(config: cfg.ViewerConfig, log_filename: Path, datapath: Path):
     """Sets up the viewer if enabled
 
     Args:
         config: the configuration to instantiate viewer
+        log_filename: the log filename to write to
+        datapath: the path to the dataset
     """
     viewer_state = ViewerState(config, log_filename=log_filename, datapath=datapath)
     banner_messages = [f"Viewer at: {viewer_state.viewer_url}"]
@@ -250,14 +244,16 @@ class ViewerState:
 
     Args:
         config: viewer setup configuration
+        log_filename: filename to log viewer output to
+        datapath: path to data
     """
 
-    def __init__(self, config: cfg.ViewerConfig, log_filename: Path, datapath: str):
+    def __init__(self, config: cfg.ViewerConfig, log_filename: Path, datapath: Path):
         self.config = config
         self.vis = None
         self.viewer_url = None
         self.log_filename = log_filename
-        self.datapath = datapath
+        self.datapath = datapath.parent if datapath.is_file() else datapath
         if self.config.launch_bridge_server:
             # start the viewer bridge server
             assert self.config.websocket_port is not None
@@ -278,11 +274,9 @@ class ViewerState:
             CONSOLE.rule(characters="=")
             CONSOLE.line()
             self.vis = Viewer(zmq_port=zmq_port, ip_address=self.config.ip_address)
-            self.vis_webrtc_thread = Viewer(zmq_port=zmq_port, ip_address=self.config.ip_address)
         else:
             assert self.config.zmq_port is not None
             self.vis = Viewer(zmq_port=self.config.zmq_port, ip_address=self.config.ip_address)
-            self.vis_webrtc_thread = Viewer(zmq_port=self.config.zmq_port, ip_address=self.config.ip_address)
 
         # viewer specific variables
         self.prev_camera_matrix = None
@@ -309,12 +303,6 @@ class ViewerState:
 
         self.output_list = None
 
-        # webrtc
-        self.pcs = set()
-        self.video_tracks = set()
-        self.webrtc_thread = None
-        self.kill_webrtc_signal = False
-
     def _pick_drawn_image_idxs(self, total_num: int) -> list[int]:
         """Determine indicies of images to display in viewer.
 
@@ -345,10 +333,8 @@ class ViewerState:
         # set the data base dir
         self.vis["renderingState/data_base_dir"].write(str(self.datapath))
 
-        # get the timestamp of the train run to set default export path name
-        timestamp_reg = re.compile("[0-9]{4}-[0-9]{2}-[0-9]{2}_[0-9]{6}")
-        timestamp_match = timestamp_reg.findall(str(self.log_filename.parents[0]))
-        self.vis["renderingState/export_path"].write(timestamp_match[-1])
+        # set default export path name
+        self.vis["renderingState/export_path"].write(self.log_filename.parent.stem)
 
         # clear the current scene
         self.vis["sceneState/sceneBox"].delete()
@@ -414,26 +400,6 @@ class ViewerState:
                 self.vis["renderingState/all_camera_paths"].write(all_path_dict)
                 self.vis["populate_paths_payload"].delete()
 
-    def _check_webrtc_offer(self):
-        """Check if there is a webrtc offer to respond to."""
-        data = self.vis["webrtc/offer"].read()
-        if data:
-            if self.webrtc_thread and self.webrtc_thread.is_alive():
-                # kill the previous thread if the webpage refreshes
-                self.kill_webrtc_signal = True
-                return
-
-            def loop_in_thread(loop):
-                asyncio.set_event_loop(loop)
-                loop.run_until_complete(self.send_webrtc_answer(data))
-
-            loop = asyncio.get_event_loop()
-            self.webrtc_thread = threading.Thread(target=loop_in_thread, args=(loop,))
-            self.webrtc_thread.daemon = True
-            self.webrtc_thread.start()
-            # remove the offer from the state tree
-            self.vis["webrtc/offer"].delete()
-
     def _update_render_aabb(self, graph):
         """
         update the render aabb box for the viewer:
@@ -497,7 +463,6 @@ class ViewerState:
 
         self._check_camera_path_payload(trainer, step)
         self._check_populate_paths_payload(trainer, step)
-        self._check_webrtc_offer()
 
         camera_object = self._get_camera_object()
         if camera_object is None:
@@ -543,7 +508,6 @@ class ViewerState:
                 is_training = self.vis["renderingState/isTraining"].read()
                 self._check_populate_paths_payload(trainer, step)
                 self._check_camera_path_payload(trainer, step)
-                self._check_webrtc_offer()
                 run_loop = not is_training
                 local_step += 1
 
@@ -663,66 +627,6 @@ class ViewerState:
 
         raise NotImplementedError
 
-    async def send_webrtc_answer(self, data):
-        """Setup the webrtc connection."""
-
-        # returns the description to for WebRTC to the specific websocket connection
-        offer = RTCSessionDescription(data["sdp"], data["type"])
-
-        if self.config.local:
-            pc = RTCPeerConnection()
-        else:
-            if self.config.skip_openrelay:
-                ice_servers = [
-                    RTCIceServer(urls="stun:stun.l.google.com:19302"),
-                ]
-            else:
-                ice_servers = [
-                    RTCIceServer(urls="stun:stun.l.google.com:19302"),
-                    RTCIceServer(urls="stun:openrelay.metered.ca:80"),
-                    RTCIceServer(
-                        urls="turn:openrelay.metered.ca:80", username="openrelayproject", credential="openrelayproject"
-                    ),
-                    RTCIceServer(
-                        urls="turn:openrelay.metered.ca:443", username="openrelayproject", credential="openrelayproject"
-                    ),
-                    RTCIceServer(
-                        urls="turn:openrelay.metered.ca:443?transport=tcp",
-                        username="openrelayproject",
-                        credential="openrelayproject",
-                    ),
-                ]
-
-            pc = RTCPeerConnection(configuration=RTCConfiguration(iceServers=ice_servers))
-        self.pcs.add(pc)
-
-        video = SingleFrameStreamTrack()
-        self.video_tracks.add(video)
-        video_sender = pc.addTrack(video)
-        print(f"viewer using video/{self.config.codec}")
-        force_codec(pc, video_sender, f"video/{self.config.codec}")
-
-        await pc.setRemoteDescription(offer)
-        answer = await pc.createAnswer()
-        await pc.setLocalDescription(answer)
-
-        self.vis_webrtc_thread["webrtc/answer"].write(
-            {"sdp": pc.localDescription.sdp, "type": pc.localDescription.type}
-        )
-        self.vis_webrtc_thread["webrtc/answer"].delete()
-
-        # continually exchange media
-        while True:
-            await asyncio.sleep(1)
-            if self.kill_webrtc_signal:
-                self.kill_webrtc_signal = False
-                return
-
-    def set_image(self, image):
-        """Write the image over webrtc."""
-        for video_track in self.video_tracks:
-            video_track.put_frame(image)
-
     def _send_output_to_viewer(self, outputs: Dict[str, Any], colors: torch.Tensor = None):
         """Chooses the correct output and sends it to the viewer
 
@@ -737,6 +641,9 @@ class ViewerState:
             if OutputTypes.RGB_FINE in self.output_list:
                 viewer_output_list.remove(OutputTypes.RGB_FINE)
             viewer_output_list.insert(0, OutputTypes.RGB)
+            # remove semantics, which crashes viewer; semantics_colormap is OK
+            if "semantics" in self.output_list:
+                viewer_output_list.remove("semantics")
             self.vis["renderingState/output_options"].write(viewer_output_list)
 
         reformatted_output = self._process_invalid_output(self.prev_output_type)
@@ -744,16 +651,22 @@ class ViewerState:
         if self.output_type_changed or self.prev_colormap_type is None:
             self.prev_colormap_type = ColormapTypes.DEFAULT
             colormap_options = []
+            self.vis["renderingState/colormap_options"].write(list(ColormapTypes))
             if outputs[reformatted_output].shape[-1] == 3:
                 colormap_options = [ColormapTypes.DEFAULT]
             if outputs[reformatted_output].shape[-1] == 1 and outputs[reformatted_output].dtype == torch.float:
-                colormap_options = list(ColormapTypes)
+                self.prev_colormap_type = ColormapTypes.TURBO
+                colormap_options = list(ColormapTypes)[1:]
             self.output_type_changed = False
             self.vis["renderingState/colormap_choice"].write(self.prev_colormap_type)
             self.vis["renderingState/colormap_options"].write(colormap_options)
         selected_output = (self._apply_colormap(outputs, colors) * 255).type(torch.uint8)
-        image = selected_output.cpu().numpy()
-        self.set_image(image)
+
+        image = selected_output[..., [2, 1, 0]].cpu().numpy()
+
+        data = cv2.imencode(".jpg", image, [cv2.IMWRITE_JPEG_QUALITY, 75])[1].tobytes()
+        data = str("data:image/jpeg;base64," + base64.b64encode(data).decode("ascii"))
+        self.vis["render_img"].write(data)
 
     def _update_viewer_stats(self, render_time: float, num_rays: int, image_height: int, image_width: int) -> None:
         """Function that calculates and populates all the rendering statistics accordingly
@@ -838,11 +751,6 @@ class ViewerState:
         if image_width > self.max_resolution:
             image_width = self.max_resolution
             image_height = int(image_width / aspect_ratio)
-        if self.config.codec != "VP8":
-            # force even values to allow hardware encoder usage
-            quantize = 2
-            image_width = int(image_width / quantize) * quantize
-            image_height = int(image_height / quantize) * quantize
         return image_height, image_width
 
     def _process_invalid_output(self, output_type: str) -> str:
@@ -872,7 +780,7 @@ class ViewerState:
         # pylint: disable=too-many-statements
         """
         Draw an image using the current camera pose from the viewer.
-        The image is sent of a TCP connection and then uses WebRTC to send it to the viewer.
+        The image is sent over a TCP connection.
 
         Args:
             graph: current checkpoint of model
